
'''
	import pathlib
	from os.path import dirname, join, normpath
	
	this_folder = pathlib.Path (__file__).parent.resolve ()
	search = normpath (join (this_folder, "../.."))

	import BODY_SCAN
	BODY_SCAN.START (
		GLOB = search + '/**/*STATUS.py'
	)
'''

import glob

import BODY_SCAN.aggregate as aggregate
import BODY_SCAN.processes.scan as scan

def START (
	GLOB = "",
	
	RELATIVE_PATH = False,
	
	MODULE_PATHS = [],
	
	SIMULTANEOUS = False,
	
	print_alarms = True,
	
	RECORDS = 0	
):
	FINDS = glob.glob (GLOB, recursive = True)
		
	if (RECORDS >= 1):
		print ()
		print ("SEARCHING FOR GLOB:")
		print ("	", GLOB)
		print ()
	
	if (RECORDS >= 1):
		print ()
		print ("	FINDS:", FINDS)
		print ("	FINDS COUNT:", len (FINDS))
		print ();
	
	def START_SIMULTANEOUSLY ():
		OUTPUT = []
	
		def FN (PATH):
			[ STATUS ] = scan.start (		
				PATH = PATH,
				MODULE_PATHS = MODULE_PATHS,
				RELATIVE_PATH = RELATIVE_PATH,
				RECORDS = RECORDS
			)
		
			return STATUS;
		
		from concurrent.futures import ThreadPoolExecutor, ProcessPoolExecutor
		with ThreadPoolExecutor () as executor:
			RETURNS = executor.map (
				FN, 
				FINDS
			)
			
			executor.shutdown (wait = True)
			
			for RETURN in RETURNS:
				OUTPUT.append (RETURN)
				
			
		return OUTPUT;
	
	
	def START_SEQUENTIALLY ():
		'''
			STARTS MULTIPLE SCANS, SEQUENTIALLY...
		'''
		PATH_STATUSES = []
		for PATH in FINDS:	
			[ STATUS ] = scan.start (		
				PATH = PATH,
				MODULE_PATHS = MODULE_PATHS,
				RELATIVE_PATH = RELATIVE_PATH,
				RECORDS = RECORDS
			)
			
			PATH_STATUSES.append (STATUS)
			
		return PATH_STATUSES;


	if (SIMULTANEOUS == True):
		PATH_STATUSES = START_SIMULTANEOUSLY ()
	else:
		PATH_STATUSES = START_SEQUENTIALLY ()

	STATUS = aggregate.start (
		PATH_STATUSES
	)


	import json
	print ("STATUS:", json.dumps (STATUS, indent = 4))
	
	def alarm_printer (paths):
		alarms = []
	
		for path in paths:
			print ("path:", path)
			print ()

			if (path ["parsed"] == False):
				alarms.append (path)

			if ("CHECKS" not in path):
				continue;
		
		
			checks = path ["CHECKS"]
		
			this_path = path ["PATH"]
			unsuccessful = []
			
			for check in checks:
				if (check ["PASSED"] == False):
					unsuccessful.append (check)
			
			if (len (unsuccessful) >= 1):
				alarms.append ({
					"path": this_path,
					"checks": unsuccessful
				})
				
		print ("alarms:", json.dumps (alarms, indent = 4))
		
	if (print_alarms):
		alarm_printer (STATUS ["PATHS"])
		
	return {
		"STATUS": STATUS
	}
	







def scan_file (path):
	with open (path, mode = 'r') as selector:
		return selector.read ()

def build_scan_string (path):
	contents = scan_file (path)
	contents += '''
		
try:
	______BODY_SCAN ["CHECKS"] = CHECKS;	
	______BODY_SCAN ["CHECKS FOUND"] = True;
except Exception as E:
	print (E)
	______BODY_SCAN ["CHECKS FOUND"] = False;
		'''

	return contents


import BODY_SCAN.functions.exceptions as bs_exceptions

import json
import time
from time import sleep, perf_counter as pc


def scan (FIND):
	# PATH = {}
	
	FINDINGS = []
	STATS = {
		"PASSES": 0,
		"ALARMS": 0
	}

	PATH_E = ""

	try:
		CONTENTS = build_scan_string (FIND)
		
		______BODY_SCAN = {}
		exec (
			CONTENTS, 
			{ 
				'______BODY_SCAN': ______BODY_SCAN,
				'__file__': FIND
			}
		)
		

		if (______BODY_SCAN ["CHECKS FOUND"] == False):
			return {
				"EMPTY": True,
				"parsed": True
			}

		
		CHECKS = ______BODY_SCAN ['CHECKS']		

		
		for CHECK in CHECKS:
			try:
				TIME_START = pc ()
				CHECKS [ CHECK ] ()
				TIME_END = pc ()
				TIME_ELAPSED = TIME_END - TIME_START

				FINDINGS.append ({
					"CHECK": CHECK,
					"PASSED": True,
					"ELAPSED": [ TIME_ELAPSED, "SECONDS" ]
				})
				
				STATS ["PASSES"] += 1
				
			except Exception as E:				
				FINDINGS.append ({
					"CHECK": CHECK,
					"PASSED": False,
					"EXCEPTION": repr (E),
					"EXCEPTION TRACE": bs_exceptions.find_trace (E)
				})
				
				STATS ["ALARMS"] += 1
		
		
		return {
			"empty": False,
			"parsed": True,
						
			"STATS": STATS,			
			"CHECKS": FINDINGS
		}
		
	except Exception as E:		
		PATH_E = E;

	return {
		"parsed": False,
		"ALARM": "An exception occurred while scanning the path.",
		"EXCEPTION": repr (PATH_E),
		"EXCEPTION TRACE": bs_exceptions.find_trace (PATH_E)
	}






'''
	import BODY_SCAN.PROCESSES.COLLECT.PATH as COLLECT_PATH
	COLLECT_PATH.FIND ()
'''
import pathlib
from os.path import dirname, join, normpath

def FIND ():
	return normpath (join (pathlib.Path (__file__).parent.resolve (), "START.PROC.PY"))



'''
	import BODY_SCAN.PROCESSES.SCAN.PATH as SCAN_PATH
	SCAN_PATH.FIND ()
'''
'''
	This returns the path of the "scan" process.
'''


import pathlib
from os.path import dirname, join, normpath

path = "python3_scan_process"

def FIND ():
	this_folder = pathlib.Path (__file__).parent.resolve ()
	return normpath (join (this_folder, path))